package peterson;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

/**
 * Maakt een connectie aan met een rechterbuur.
 * @author Pieter van Ede - 3019764 <br /> Wesley Kok - 3019853
 *
 */
public class ClientConnectionMaker
{	
	private Socket opvolger;
	
	/**
	 * Maakt een verbinding met rechterbuur en legt alvast een deel van de verbinding met 
	 * die buur aan.
	 * 
	 * @param poortOpvolger  poort op de localhost waarmee verbinding moet worden gemaakt
	 */
	public ClientConnectionMaker (int poortOpvolger)
	{
		boolean verbonden = false;
		this.opvolger = null;
		for(int i=0; i<GP2.retry && !verbonden; i++)
		{
			try
			{
				opvolger = new Socket("localhost", poortOpvolger);
				verbonden = true;
			}
			catch(IOException e)
			{
				Util.printDebug("Verbinding maken met rechterbuur " +
							"is mislukt. (ClientConnectionMaker) \n" + e.getMessage(), false);
				try
				{	Thread.sleep(500);	}
				catch(InterruptedException ee)
				{	/* we gaan even slapen om zometeen te kijken of onze volgende buur al bestaat*/	}
			}
		}
		if (!verbonden)
			Util.printDebug("Time-out met verbinden met rechterbuur. (ClientConnectionMaker)", true);
	}
	
	/**
	 * Retourneert het kanaal van de server (ofwel rechterbuur), waarmee deze client verbonden is.
	 * 
	 * @return  de kanaal waarover met de rechterbuur gecommuniceerd kan worden
	 * @throws IOException  als er iets fout gaat in het aanmaken van het kanaal
	 */
	public Kanaal nieuweBuur() throws IOException
	{
		ObjectInputStream volgendeIn = new ObjectInputStream(opvolger.getInputStream());
		ObjectOutputStream volgendeUit = new ObjectOutputStream(opvolger.getOutputStream());
		
		return new Kanaal(volgendeUit, volgendeIn, opvolger);
	}
	
	/**
	 * Retourneert het kanaal van de server (ofwel rechterbuur), waarmee deze client verbonden is.
	 * Maar er kan alleen verstuurd worden over dit kanaal.
	 * 
	 * @return  de kanaal waarover met de rechterbuur gecommuniceerd kan worden
	 * @throws IOException  als er iets fout gaat in het aanmaken van het kanaal
	 */
	public Kanaal eenrichtingBuur() throws IOException
	{
		ObjectOutputStream volgendeUit = new ObjectOutputStream(opvolger.getOutputStream());
		ObjectInputStream volgendeIn = new ObjectInputStream(opvolger.getInputStream());
		return new Kanaal(volgendeUit, volgendeIn, opvolger);
	}
}