package peterson;

import java.net.ServerSocket;
import java.net.SocketException;

/**
 * Met deze klasse wordt het programma begonnen. Het ontleedt de parameters en start de electie.
 * @author Pieter van Ede - 3019764 <br /> Wesley Kok - 3019853
 *
 */
public class GP2 
{
	/**
	 * Hiermee kan je bepalen hoevaak een proces moet proberen verbinding te maken met
	 * het volgende proces in de ring, voordat hij de handdoek in de ring gooit.
	 */
	final static int retry = 25;
	
	/**
	 * Hiermee kan je bepalen of er debug-statements mogen worden geprint. Voornamelijk van
	 * toepassing op de meldingen bij try-catch blokken.
	 */
	final static boolean debug = true;
	
	/**
	 * Bepaald of het programma op een enter moet wachten, zodat in het geval het programma
	 * in een apart venster wordt gedraaid, ook de input zichtbaar kan laten zijn.
	 */
	final static boolean zelfdeVenster = false;
	
	/**
	 * Hiermee begint het programma.
	 * Eerst worden de parameters gecontroleerd en geparsed.
	 * Hierna wordt de electie gestart.
	 * 
	 * @param args  parameters van het programma <br />
	 * 		- int poort <br />
	 * 		- int poortOpvolger <br />
	 * 		- int id
	 */
	public static void main (String[] args)
	{
		//	parsen van de parameters
		int poortTemp = 0;
		int poortOpvolgerTemp = 0;
		int idTemp = 0;
		try 
		{
            idTemp            = Integer.valueOf(args[0]).intValue();
            poortTemp         = Integer.valueOf(args[1]).intValue();
            poortOpvolgerTemp = Integer.valueOf(args[2]).intValue();
		} 
		catch(NumberFormatException e) 
		{
			Util.printError(idTemp + ": Een van de parameters was geen nummer.", true);
        }
		catch(ArrayIndexOutOfBoundsException e)
		{
			Util.printError(idTemp + ": Er zijn te weinig parameters meegegeven.", true);
		}
		catch(Exception e)
		{
			Util.printError("Onverwachtte fout: " + e.getMessage(), true);
		}
		if (args.length > 3)
		{
			Util.printError(idTemp + ": Er zijn te veel parameters meegegeven.", true);
		}
		
		//	we starten nu de electie
		Util.printDebug(idTemp + ": Ik begin mijn verkiezingsproces.", false);
		GP2.startElectie(poortTemp, poortOpvolgerTemp, idTemp);
		Util.printDebug(idTemp + ": Het programma is correct afgelopen.", true);
	}
	
	/**
	 * Start van electie. Start serverthread van proces en de electie.
	 * Hiervoor moeten de gedeelde queue en semafoor ook aangemaakt worden en meegegeven.
	 * 
	 * @param poort  poort op de localhost waarop dit proces bereikbaar is
	 * @param poortOpvolger  poort op de localhost van de rechterbuur in de originele ringconfiguratie
	 * @param id  identitiet van dit proces
	 */
	private static void startElectie(int poort, int poortOpvolger, int id)
	{
		try
		{
			Semafoor beschikSockets = new Semafoor(0);
			Queue sockets = new Queue();
			ServerConnectionMaker serverThread = new ServerConnectionMaker(poort, beschikSockets, sockets);
			serverThread.start();
			ServerSocket server = serverThread.getServer();
			Electie electieAlg = new Electie(poort, poortOpvolger, id, beschikSockets, sockets);
			electieAlg.startElectie();
			
			//zorg ervoor dat de serverthread doodgaat
			server.close();
		}
		catch(SocketException se)
		{
			//dit kan zijn, want misschien is de server nog in een accept-fase.
			//echter die zal nooit voltooien, dus mogen de stakkers klagen
		}
		catch(Exception e)
		{
			Util.printError(id + ": Fout in electie-algoritme: \n" + e.getMessage(), true);
		}
	}
}