package peterson;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

/**
 * Dit object is een kanaal tussen 2 processen.
 * Het regelt het sturen en ontvangen van pakketjes tussen hen.
 * @author Pieter van Ede - 3019764 <br /> Wesley Kok - 3019853
 *
 */
public class Kanaal 
{
	private ObjectOutputStream oos;
	private ObjectInputStream ois;
	private Socket s;
	
	/**
	 * Maakt een nieuw kanaal aan met gegeven ObjectOutputStream en ObjectInputStream.
	 * 
	 * @param oos  ObjectOutputStream waarover objecten naar de buur gestuurd kunnen worden
	 * @param ois  ObjectInputStream waarover objecten van de buur gelezen kunnen worden
	 */
	public Kanaal (ObjectOutputStream oos, ObjectInputStream ois, Socket s)
	{
		this.oos = oos;
		this.ois = ois;
		this.s = s;
	}
	
	/**
	 * Stuurt een pakketje naar de buur.
	 * 
	 * @param p  pakketje dat gestuurd moet worden
	 * @throws IOException  als er iets foutgaat in het versturen van het pakketje
	 */
	public void stuur (Pakketje p) throws IOException
	{
		oos.writeObject(p);
		oos.flush();
	}
	
	/**
	 * Ontvangt een pakketje van de buur
	 * 
	 * @return  het ontvangen pakketje
	 * @throws IOException  als er iets foutgaat in het ontvangen van het pakketje
	 * @throws ClassNotFoundException  als het object-type/declaratie van het ontvangen 
	 * object (dit zou Pakketje moeten zijn) niet gevonden kan worden
	 */
	public Pakketje ontvang () throws IOException, ClassNotFoundException
	{
		return (Pakketje) ois.readObject();
	}
	
	/**
	 * Sluit het kanaal voor de aanroeper. Dit betekent dat degene aan de andere kant van dit
	 * kanaal (die natuurlijk ook een kanaal-object heeft) zit, zijn kanaal nog wel als open
	 * ziet.
	 * 
	 * @throws IOException  als er iets foutgaat in het sluiten van het kanaal
	 */
	public void sluitKanaal () throws IOException
	{
		//Streams sluiten
		this.oos.close();
		if(ois != null)
			this.ois.close();
		
		//Socket sluiten
		this.s.close();
	}
}