package peterson;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * De thread die connecties maakt met linkerburen.
 * Als een connectie is gemaakt, stopt hij deze in de, met het electieproces gedeelde, queue en 
 * doet hij een vrij operatie op de, eveneens met het electieproces gedeelde, semafoor.
 * Nu weet het proces dat er een nieuwe client is en kan hij deze uit de queue halen.
 * @author Pieter van Ede - 3019764 <br /> Wesley Kok - 3019853
 */
public class ServerConnectionMaker extends Thread
{
	private Semafoor beschikbareKanalen;
	private int poort;
	private Queue sockets;
	private ServerSocket serverSocket;
	
	/**
	 * Maakt een object aan waarmee connecties met clients kunnen worden geaccepteerd,
	 * ofwel waarmee connecties met linkerburen kan worden gemaakt.
	 * 
	 * @param poort  de poort waarna deze server aan de linkerzijde luistert/bereikbaar is
	 */
	public ServerConnectionMaker(int poort, Semafoor beschikbareKanalen, Queue sockets)
	{
		this.beschikbareKanalen = beschikbareKanalen;
		this.poort = poort;
		this.sockets = sockets;
		
		//maak de server
		serverSocket = makeServer();
	}
	
	/**
	 * Zodra de thread gestart wordt, gaat hij, zolang het algoritme niet is geeindigd,
	 * nieuwe clienten accepteren en deze in een, met het proces, gedeelde queue.
	 */
	public void run()
	{
		//in een eindige loop clients accepteren en in queue stoppen. Hij stopt op het moment
		//dat het programma, na de electie, de serversocket sluit.
		while (!serverSocket.isClosed())
		{
			sockets.enqueue(nieuweBuur(serverSocket));
			beschikbareKanalen.v();
		}
	}
	
	/**
	 * Zet de server op.
	 * 
	 * @return  de socket waar de server te vinden is
	 */
	private ServerSocket makeServer()
	{
		serverSocket = null;
		try 
		{	serverSocket = new ServerSocket(poort);	} 
		catch (IOException e) 
		{
			Util.printDebug("Serversocketmaken is mislukt. \n" + e.getMessage(), true);
		}
		return serverSocket;
	}
	
	/**
	 * Retourneert de serversocket van deze thread.
	 * 
	 * @return  de serversocket van deze thread
	 */
	protected ServerSocket getServer()
	{
		return this.serverSocket;
	}
	
	/**
	 * Accepteert de verbinding met een nieuwe client.
	 * 
	 * @return  De nieuwe socket waarmee de server in contact is gekomen.
	 * @throws IOException
	 */
	private Kanaal nieuweBuur(ServerSocket serverSocket)
	{
		Kanaal kanaal = null;
		try
		{
			Socket clientSocket = serverSocket.accept();
			kanaal = new Kanaal( new ObjectOutputStream(clientSocket.getOutputStream()) , 
					new ObjectInputStream(clientSocket.getInputStream()), clientSocket );
		}
		catch (IOException e)
		{
			if(!serverSocket.isClosed())
				Util.printDebug("Clientsocketmaken is mislukt. \n" + e.getMessage(), true);
		}
		return kanaal;
	}
}